@@ -494,8 +494,8 @@ class DicomReaderWriter(object):
     rd_dictionary: Dict[str, RDBase]
     rp_dictionary: Dict[str, PlanBase]
     rois_in_index_dict: Dict[int, List[str]]  # List of rois at any index
-    dicom_handle = sitk.Image
-    dose_handle: sitk.Image
+    dicom_handle: sitk.Image or None
+    dose_handle: sitk.Image or None
     annotation_handle: sitk.Image or None
     all_rois: List[str]
     rois_in_loaded_index: List[str]
@@ -530,6 +530,11 @@ class DicomReaderWriter(object):
         :param index: index to reference series_instances_dictionary, default 0
         :param series_instances_dictionary: dictionary of series instance UIDs of images and RTs
         """
+        self.verbose = verbose
+        self.annotation_handle = None
+        self.dicom_handle = None
+        self.dose_handle = None
+        self.rois_in_index_dict = {}
         self.rt_dictionary = {}
         self.mask_dictionary = {}
         self.plan_pydicom_string_keys = plan_pydicom_string_keys
@@ -568,11 +573,9 @@ class DicomReaderWriter(object):
         self.reader.MetaDataDictionaryArrayUpdateOn()
         self.reader.LoadPrivateTagsOn()
         self.reader.SetOutputPixelType(sitk.sitkFloat32)
-        self.verbose = verbose
         self.dicom_handle_uid = None
         self.dicom_info_uid = None
         self.RS_struct_uid = None
-        self.annotation_handle = None
         self.mask = None
         self.rd_study_instance_uid = None
         self.index = index
@@ -584,7 +587,10 @@ class DicomReaderWriter(object):
 
     def set_index(self, index: int):
         self.index = index
-        self.__check_contours_at_index__(index)
+        if self.index in self.rois_in_index_dict:
+            self.rois_in_loaded_index = self.rois_in_index_dict[self.index]
+        else:
+            self.rois_in_loaded_index = []
 
     def __compile__(self):
         """
@@ -761,11 +767,16 @@ class DicomReaderWriter(object):
                 self.series_instances_dictionary[index] = template
         self.__check_if_all_contours_present__()
 
+    def __mask_empty_mask__(self) -> None:
+        if self.dicom_handle:
+            self.image_size_cols, self.image_size_rows, self.image_size_z = self.dicom_handle.GetSize()
+            self.mask = np.zeros(
+                [self.dicom_handle.GetSize()[-1], self.image_size_rows, self.image_size_cols, len(self.Contour_Names) + 1],
+                dtype=np.int8)
+            self.annotation_handle = sitk.GetImageFromArray(self.mask)
+
     def __reset_mask__(self):
-        if self.mask is not None:
-            self.mask = None
-        if self.annotation_handle is not None:
-            self.annotation_handle = None
+        self.__mask_empty_mask__()
         self.mask_dictionary = {}
 
     def __reset__(self):
@@ -793,6 +804,10 @@ class DicomReaderWriter(object):
             self.__set_associations__(associations=associations)
         if check_contours:  # I don't want to run this on the first build..
             self.__check_if_all_contours_present__()
+        if contour_names is not None or self.associations is not None:
+            if self.verbose:
+                print("Contour names or associations changed, resetting mask")
+            self.__reset_mask__()
 
     def __set_associations__(self, associations: List[ROIAssociationClass] = None):
         if associations is not None:
@@ -863,7 +878,6 @@ class DicomReaderWriter(object):
 
     def __check_if_all_contours_present__(self):
         self.indexes_with_contours = []
-        self.rois_in_index_dict = {}
         for index in self.series_instances_dictionary:
             self.__check_contours_at_index__(index)
             self.rois_in_index_dict[index] = self.rois_in_loaded_index
@@ -1283,12 +1297,13 @@ class DicomReaderWriter(object):
             if self.verbose:
                 print('Loading images for {} at \n {}\n'.format(self.series_instances_dictionary[index].Description,
                                                                 self.series_instances_dictionary[index].path))
-                print("Erasing previous masks loaded for different image sets")
-            self.__reset_mask__()
             dicom_names = self.series_instances_dictionary[index].files
             self.ds = pydicom.read_file(dicom_names[0])
             self.reader.SetFileNames(dicom_names)
             self.dicom_handle = self.reader.Execute()
+            if self.verbose:
+                print("Erasing any previous mask as we load a new new image set")
+            self.__reset_mask__()
             add_sops_to_dictionary(sitk_dicom_reader=self.reader,
                                    series_instances_dictionary=self.series_instances_dictionary)
             if max(self.flip_axes):
@@ -1338,12 +1353,6 @@ class DicomReaderWriter(object):
             output.SetOrigin(origin)
             self.dose_handle = output
 
-    def __mask_empty_mask__(self) -> None:
-        if self.dicom_handle:
-            self.mask = np.zeros(
-                [self.dicom_handle.GetSize()[-1], self.image_size_rows, self.image_size_cols, len(self.Contour_Names) + 1],
-                dtype=np.int8)
-
     def __characterize_RT__(self, RT: RTBase):
         if self.RS_struct_uid != RT.SeriesInstanceUID:
             self.structure_references = {}
@@ -1373,7 +1382,6 @@ class DicomReaderWriter(object):
             print('Loading images for index {}, since mask was requested but image loading was '
                   'previously different\n'.format(index))
             self.get_images()
-        self.__mask_empty_mask__()
         RTs = self.series_instances_dictionary[index].RTs
         for RT_key in RTs:
             RT = RTs[RT_key]
