@@ -1,6 +1,6 @@
 __author__ = 'Brian M Anderson'
 # Created on 10/28/2020
-from .ReaderWriter import DicomReaderWriter, Dicom_to_Imagestack, plot_scroll_Image, sitk
+from .ReaderWriter import DicomReaderWriter, plot_scroll_Image, sitk
 
 
 if __name__ == '__main__':
